<?php

/*
  Cartproductfeed Transaction Controller
    Handles AJAX requests
	  Copyright 2014 Purple Turtle Productions. All rights reserved.
		license	GNU General Public License version 3 or later; see GPLv3.txt
  By: Keneto
  2014-06-05
 */

defined('_JEXEC') or die;

//The transaction AJAX should never need to use the define and it's causing errors
//  Following steps to retrieve it programmatically will cost extra cycles when the
//  AJAX call needs to be lightning fast
define('FEED_PLUGIN_VERSION', '' );

require_once dirname(__FILE__) . '/../core/data/feedcore.php';
require_once dirname(__FILE__) . '/../core/classes/dialogbasefeed.php';
require_once dirname(__FILE__) . '/../core/data/attributesfound.php';

class CartproductfeedControllerTransaction extends JControllerAdmin {

	public function fetchCategory() {

		//Client is typing letters under taxomony and would like to see similar terms listed
		$data = file_get_contents(dirname(__FILE__) . '/../core/feeds/' . strtolower($_POST['service_name']) . '/categories.txt');
		$data = explode("\n", $data);
		$searchTerm = strtolower($_POST['partial_data']);
		$count = 0;
		$canDisplay = true;
		foreach($data as $this_item) {
			if (strlen($this_item) * strlen($searchTerm) == 0)
				continue;

			if (strpos(strtolower($this_item), $searchTerm) !== false) {

				//Transform item from chicken-scratch into something the system can recognize later
				$option = str_replace(" & ", ".and.", str_replace(" / ", ".in.", trim($this_item)));

				//Transform a category from chicken-scratch into something the user can read
				$text = htmlentities(trim($this_item));

				if ($canDisplay)
					echo '<div class="categoryItem" onclick="doSelectCategory(this, \'' . $option . '\')">' . $text . '</div>';

				$count++;
				if ((strlen($searchTerm) < 3) && ($count > 15))
					$canDisplay = false;
			}
		}

		if ($count == 0)
			echo '<div class="categoryItem">No matching categories found</div>';

		if (!$canDisplay)
			echo '<div class="categoryItem">(' . $count . ' results)</div>';

		JFactory::getApplication()->close();
	}

	public function fetchLocalCategories() {

		$input = JFactory::getApplication()->input;
		$shopID = $input->getInt('shop_id', 0);

		global $pfcore;

		//shopID is used by the RapidCart Source
		if ($shopID > 0) {
			$pfcore->shopID = $shopID;
			//Need ShopName for URL generation
			$db = JFactory::getDBO();
			$db->setQuery('SELECT name FROM #__rapidcart_shops WHERE id=' . (int) $shopID);
			$pfcore->shopName = $db->loadResult();
		}

		require_once dirname(__FILE__) . '/../core/data/productcategories.php';

		$categoryList = new PProductCategories();
		$result = new stdClass();
		$result->children = array();

		foreach($categoryList->categories as $this_category)
			if (!isset($this_category->parent_category))
				$this->process_category($result->children, $this_category);
		
		echo json_encode($result);

		JFactory::getApplication()->close();

	}

	public function fetchSelectFeed() {

		$input = JFactory::getApplication()->input;
		$feedtype = $input->post->get('feedtype', '', 'string');

		if (strlen($feedtype) == 0) {
			JFactory::getApplication()->close();
			return;
		}

		global $pfcore;

		if ($pfcore->callSuffix == 'JS') {
			$user	= JFactory::getUser();
			$db = JFactory::getDBO();
			$db->setQuery('SELECT id FROM #__rapidcart_shops WHERE created_by=' . $user->id);
			$pfcore->shopID = $db->loadResult();
		}

		$inc = dirname(__FILE__) . '/../core/feeds/' . strtolower($feedtype) . '/dialognew.php';
		$feedObjectName = $feedtype . 'Dlg';

		include_once $inc;
		$f = new $feedObjectName();
		echo $f->mainDialog();

		JFactory::getApplication()->close();

	}

	function doOutput($output) {
		ob_clean();
		echo json_encode($output);
		JFactory::getApplication()->close();
	}

	public function getFeed() {

		ob_start(null);

		$input = JFactory::getApplication()->input;
		$requestCode = $input->post->get('provider', '', 'string');
		$local_category =	$input->post->get('local_category', '', 'string');
		$remote_category = $input->post->get('remote_category', '', 'string');
		$file_name = $input->getString('file_name', '');
		$feedIdentifier = $input->post->get('feed_identifier', '', 'string');
		$shopID = $input->getInt('shop_id', 0);
		$saved_feed_id = $input->getInt('feed_id', 0);

		if (strlen($file_name) == 0)
			$file_name = 'feed' . rand(10, 1000);

		$output = new stdClass();
		$output->url = '';

		if (strlen($requestCode) * strlen($local_category) == 0) {
			$output->errors = 'Error in AJAX request. Insufficient data';
			$this->doOutput($output);
			return;
		}
		if (strlen($remote_category) == 0) {
			$output->errors = 'Error: Insufficient data. Please fill in "' . $requestCode . ' category"';
			$this->doOutput($output);
			return;
		}

		global $pfcore;

		//shopID is used by the RapidCart Source
		if ($shopID > 0) {
			$pfcore->shopID = $shopID;
			if (strpos($file_name, $shopID . '-') !== 0)
				$file_name = $shopID . '-' . $file_name;
			//Need ShopName for URL generation
			$db = JFactory::getDBO();
			$db->setQuery('SELECT name FROM #__rapidcart_shops WHERE id=' . (int) $shopID);
			$pfcore->shopName = $db->loadResult();
		}

		require_once dirname(__FILE__) . '/../core/classes/md5.php';
		require_once dirname(__FILE__) . '/../core/classes/cart-product-feed.php';
		require_once dirname(__FILE__) . '/../core/classes/providerlist.php';
		require_once dirname(__FILE__) . '/../core/data/attributedefaults.php';
		require_once dirname(__FILE__) . '/../core/data/feedactivitylog.php';
		require_once dirname(__FILE__) . '/../core/data/feedcore.php';
		require_once dirname(__FILE__) . '/../core/data/productcategories.php';
		require_once dirname(__FILE__) . '/../core/data/feedfolders.php';
		require_once dirname(__FILE__) . '/../core/data/feedoverrides.php';
		require_once dirname(__FILE__) . '/../core/data/productlist.php';
		require_once dirname(__FILE__) . '/../core/data/shippingdata.php';
		require_once dirname(__FILE__) . '/../core/data/taxationdata.php';
		require_once dirname(__FILE__) . '/../core/registration.php';

		$dir = PFeedFolder::uploadFolder();
		if (!is_dir($dir))
			mkdir($dir);

		$providerFile = dirname(__FILE__) . '/../core/feeds/' . strtolower($requestCode) . '/feed.php';
		if (!file_exists($providerFile)) {
			$output->errors = 'Provider not valid in request code';
			$this->doOutput($output);
			return;
		}

		require_once $providerFile;

		$saved_feed = null;
		if ( (strlen($saved_feed_id) > 0) && ($saved_feed_id > -1) ) {
			require_once dirname(__FILE__) . '/../core/data/savedfeed.php';
			$saved_feed = new PSavedFeed($saved_feed_id);
		}

		$providerClass = 'P' . $requestCode . 'Feed';
		$x = new $providerClass;
		if (strlen($feedIdentifier) > 0)
			$x->activityLogger = new PFeedActivityLog($feedIdentifier);
		$x->getFeedData($local_category, $remote_category, $file_name, $saved_feed);

		if ($x->success)
			$output->url = PFeedFolder::uploadURL() . $x->providerName . '/' . $file_name . '.' . $x->fileformat;
		$output->errors = $x->message;
		$this->doOutput($output);

	}

	public function getFeedStatus() {

		global $pfcore;

		$pfcore->shopID = -1; //-1 = Don't care

		$input = JFactory::getApplication()->input;
		$feedIdentifier = $input->post->get('feed_identifier', '', 'string');
		echo $pfcore->settingGet('cp_feedActivity_' . $feedIdentifier);
		JFactory::getApplication()->close();

	}

	public function mappingsErase() {

		$input = JFactory::getApplication()->input;
		$providerName = $input->post->get('service_name', '', 'string');

		$db = JFactory::getDBO();
		$db->setQuery("
		DELETE
		FROM #__cartproductfeed_options 
		WHERE (name like '" . $providerName . "_cp_%') AND (kind = 0)");
		$db->query();
		echo "1";
		JFactory::getApplication()->close();

	}

	private function process_category(&$target_list, $this_category) {
		//Called by fetchLocalCategories()
		$new_category = new stdClass();
		$new_category->id = $this_category->id;
		$new_category->title = $this_category->title;
		$new_category->tally = $this_category->tally;
		$new_category->children = array();
		$target_list[] = $new_category;
		foreach($this_category->children as $child)
			$this->process_category($new_category->children, $child);
	}

	public function setAttributeOption() {

		//Save a change in attribute mappings
		$input = JFactory::getApplication()->input;
		$service_name = $input->getString('service_name', '');
		$attribute = $input->getString('attribute', '');
		$mapto = $input->getString('mapto', '');
		$date	= JFactory::getDate();
		$user	= JFactory::getUser();
		$option_name = $service_name . '_cp_' . $attribute;

		//Does the option exist?
		$db = JFactory::getDBO();
		$db->setQuery('SELECT id, ordering FROM #__cartproductfeed_options WHERE name = ' . $db->quote( $db->escape($option_name), false));
		$db->execute();
		$r = $db->loadObject();
		if ($r)
			$order = $r->ordering;
		else {
			$db->setQuery('SELECT COUNT(*) FROM #__cartproductfeed_options');
			$db->query();
			$order = $db->loadResult();
		}

		$newdata = new stdClass();
		$newdata->name = $option_name;
		$newdata->value = $mapto;
		$newdata->kind = 0;
		$newdata->ordering = $order;
		$newdata->created = $date->toSql();
		$newdata->created_by = $user->get('id');
		$newdata->catid = 0;
		$newdata->modified = $date->toSql();
		$newdata->modified_by = $user->get('id');
		if ($r)
			$newdata->id = $r->id;
		$db->insertObject('#__cartproductfeed_options', $newdata, 'id');

		JFactory::getApplication()->close();

	}

	public function setAttributeUserMap() {

		//Save a change in attribute mappings: advanced mode uses one field to map all attributes
		//should be more robust since there are now more mappings
		$input = JFactory::getApplication()->input;
		$service_name = $input->getString('service_name', '');
		$attribute = $input->post->get('attribute', '', 'string');
		$mapto = $input->post->get('mapto', '', 'string');
		$shopID = $input->getInt('shop_id', 0);

		$date	= JFactory::getDate();
		$user	= JFactory::getUser();

		global $pfcore;

		//shopID is used by the RapidCart Source
		if ($shopID > 0)
			$pfcore->shopID = $shopID;

		$map_string = $pfcore->settingGet('cpf_attribute_user_map_' . $service_name);
		if (strlen($map_string) == 0)
			$map = array();
		else {
			$map = json_decode($map_string);
			$map = get_object_vars($map);
		}

		$map[$mapto] = $attribute;

		$pfcore->settingSet('cpf_attribute_user_map_' . $service_name, json_encode($map));

		JFactory::getApplication()->close();
	}

	public function setToken() {

		$input = JFactory::getApplication()->input;
		$strRapidCartToken = $input->getString('edtRapidCartToken', '');
		if (strlen($strRapidCartToken) == 0)
			$strRapidCartToken = rand(1000, 9999) . '-' . date('U') . '-' . rand(1000, 9999);

		global $pfcore;

		$pfcore->settingSet('cp_rapidcarttoken', $strRapidCartToken);

		$this->setRedirect('index.php?option=com_cartproductfeed');

	}

	public function updateAllFeeds() {
		echo 'Not implemented';
		JFactory::getApplication()->close();
	}

	public function updateSetting() {

		require_once dirname(__FILE__) . '/../core/data/savedfeed.php';

		//Save settings from the advanced options box or other (eg cp_feed_timeout)
		$input = JFactory::getApplication()->input;
		$setting = $input->getString('setting', '');
		$value = $input->getString('value', '');
		$shopID = $input->getInt('shop_id', 0);
		$feedID = $input->getInt('feedid', 0);
		$date	= JFactory::getDate();
		$user	= JFactory::getUser();
		$target = '';

		global $pfcore;
		if ($shopID > 0)
			$pfcore->shopID = $shopID;

		//Some PHPs don't return the post correctly when it's long data
		if (strlen($setting) == 0) {
			$setting = substr(file_get_contents("php://input"), 8);
			$indexOfAmp = strpos($setting, '&value');
			if ($indexOfAmp !== false)
				$setting = substr($setting, 0, $indexOfAmp);
		}

		//unmangle the "setting" value since wordpress-compatibility mangled it
		if (strpos($setting, 'cp_advancedFeedSetting') !== false) {

			//$value may get truncated on an & because $_POST can't parse
			//so pull value manually
			$postdata = file_get_contents("php://input");
			$i = strpos($postdata, '&');
			if ($i !== false)
				$postdata = substr($postdata, $i + 7);

			//Strip the provider name out of the setting
			$target = substr($setting, strpos($setting, '-') + 1);
			$setting = $target  . '-cart-product-settings';

			if ($feedID == 0) {
				$pfcore->settingSet($setting, $value);
			} else {
				$saved_feed = new PSavedFeed($feedID);
				$saved_feed->save_ownoverrides($value);
			}

		} else
			$pfcore->settingSet($setting, $value);

		//if (strpos($setting, 'cp_advancedFeedSetting') !== false) {

		/*
		//Does the option exist?
		$db = JFactory::getDBO();
		$db->setQuery('SELECT id, ordering FROM #__cartproductfeed_options WHERE name = ' . $db->quote( $setting));
		$r = $db->loadObject();
		if ($r)
			$order = $r->ordering;
		else {
			$db->setQuery('SELECT COUNT(*) FROM #__cartproductfeed_options');
			$db->query();
			$order = $db->loadResult();
		}

		//Initialize a database object that may-or-may-not be used
		$newdata = new stdClass();
		$newdata->name = $setting;
		$newdata->value = $value;
		$newdata->kind = 0;

		$newdata->created = $date->toSql();
		$newdata->created_by = $user->get('id');
		$newdata->catid = 0;
		$newdata->modified = $date->toSql();
		$newdata->modified_by = $user->get('id');
		if ($shopID > 0)
			$newdata->shop_id = $shopID;
		$saveDatabaseObject = false;

		if (strlen($target) > 0) {

			//Configure the database object
			$newdata->name = $target  . '-cart-product-settings';
			$newdata->kind = 1;
			$newdata->value = $postdata;
			$saveDatabaseObject = true;

		}

		//if it's been marked okay, send the database object out
		if ($saveDatabaseObject) {
			if ($r) {
				$newdata->id = $r->id;
				$db->updateObject('#__cartproductfeed_options', $newdata, 'id');
			} else {
				$newdata->ordering = $order;
				$db->insertObject('#__cartproductfeed_options', $newdata, 'id');
			}
		}*/

		//Cleanup
		echo 'Updated.';
		JFactory::getApplication()->close();

	}

}