<?php

/***************************************************
  Cartproductfeed - Feeds Model
	  Copyright 2014 Purple Turtle Productions. All rights reserved.
		license	GNU General Public License version 3 or later; see GPLv3.txt
    2014-06-16 by Keneto
 ***************************************************/
 
// No direct access to this file
defined('_JEXEC') or die('Restricted access');
 
// import Joomla modellist library
//jimport('joomla.application.component.modellist');

class CartproductfeedModelFeeds extends JModelList {

  protected $msg = '';
		
  public function __construct($config = array())
  {
    if (empty($config['filter_fields']))
	{
	  $config['filter_fields'] = array(
	    'id', 'a.id',
		'title', 'a.title',
		'category', 'c.title',
		'created', 'a.created',
		'created_by', 'a.created_by',
		'ordering', 'a.ordering',
		'language', 'a.language',
		'sortorder', 'a.ordering',
		'product_count', 'a.product_count',
	  );
	}

    parent::__construct($config);
  }
  
  protected function populateState($ordering = null, $direction = null)
  {
	// Load the filter state.
	$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
	$this->setState('filter.search', $search);

	$accessId = $this->getUserStateFromRequest($this->context . '.filter.access', 'filter_access', null, 'int');
	$this->setState('filter.access', $accessId);

	$published = $this->getUserStateFromRequest($this->context . '.filter.state', 'filter_state', '', 'string');
	$this->setState('filter.state', $published);

	$categoryId = $this->getUserStateFromRequest($this->context . '.filter.category_id', 'filter_category_id', '');
	$this->setState('filter.category_id', $categoryId);

	$language = $this->getUserStateFromRequest($this->context . '.filter.language', 'filter_language', '');
	$this->setState('filter.language', $language);

	$tag = $this->getUserStateFromRequest($this->context . '.filter.tag', 'filter_tag', '');
	$this->setState('filter.tag', $tag);

	// Load the parameters.
	$params = JComponentHelper::getParams('com_cartproductfeed');
	$this->setState('params', $params);

	// List state information.
	parent::populateState('a.title', 'asc');
  }

	public function getItems() {
		require_once dirname(__FILE__) . '/../core/classes/providerlist.php';
		$result = parent::getItems();
		$providerList = new PProviderList();
		foreach ($result as $item)
			$item->typeName = $providerList->getPrettyNameByType($item->type);
		return $result;
	}

  protected function getListQuery()
  {
	// Create a new query object.
	$db = $this->getDbo();
	$query = $db->getQuery(true);
	$user = JFactory::getUser();

	// Select the required fields from the table.
	$query->select(
		$this->getState(
			'list.select',
			'a.id, a.title, a.description, a.checked_out, a.checked_out_time, a.catid,' .
				'a.url, a.product_count, a.own_overrides, a.feed_overrides, a.state, a.access, a.ordering, a.created_by, a.created, a.type,
				a.language, a.publish_up, a.publish_down'
		)
	);
	$query->from($db->quoteName('#__cartproductfeed_feeds') . ' AS a');

	// Join over the language
	$query->select('l.title AS language_title')
		->join('LEFT', $db->quoteName('#__languages') . ' AS l ON l.lang_code = a.language');

	// Join over the users for the checked out user.
	$query->select('uc.name AS editor')
		->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

	// Join over the asset groups.
	$query->select('ag.title AS access_level')
		->join('LEFT', '#__viewlevels AS ag ON ag.id = a.access');

	// Join over the categories.
	$query->select('c.title AS category_title')
		->join('LEFT', '#__categories AS c ON c.id = a.catid');

	global $pfcore;
	if ($pfcore->callSuffix == 'JS') {
		// Join over shop
		$query->select('shop.created_by AS shop_owner')
			->join('LEFT', '#__rapidcart_shops AS shop ON shop.id = a.shop_id');
		//Filter over shop-owner
		$query->where('shop.created_by = ' . $user->id);
	}

	// Filter by access level.
	if ($access = $this->getState('filter.access'))
	{
		$query->where('a.access = ' . (int) $access);
	}

	// Implement View Level Access
	if (!$user->authorise('core.admin'))
	{
		$groups = implode(',', $user->getAuthorisedViewLevels());
		$query->where('(a.access IN (' . $groups . '))');
	}

	// Filter by published state
	$published = $this->getState('filter.state');
	if (is_numeric($published))
	{
		$query->where('a.state = ' . (int) $published);
	}
	elseif ($published === '')
	{
		$query->where('(a.state IN (0, 1))');
	}

	// Filter by category.
	$categoryId = $this->getState('filter.category_id');
	if (is_numeric($categoryId))
	{
		$query->where('a.catid = ' . (int) $categoryId);
	}

	// Filter by search in title
	$search = $this->getState('filter.search');
	if (!empty($search))
	{
		if (stripos($search, 'id:') === 0)
		{
			$query->where('a.id = ' . (int) substr($search, 3));
		}
		else
		{
			$search = $db->quote('%' . $db->escape($search, true) . '%');
			$query->where('(a.title LIKE ' . $search . ' OR a.title LIKE ' . $search . ')');
		}
	}

	// Filter on the language.
	if ($language = $this->getState('filter.language'))
	{
		$query->where('a.language = ' . $db->quote($language));
	}

	// Add the list ordering clause.
	$orderCol = $this->state->get('list.ordering');
	$orderDirn = $this->state->get('list.direction');
	if ($orderCol == 'a.ordering' || $orderCol == 'category_title')
	{
		$orderCol = 'c.title ' . $orderDirn . ', a.ordering';
	}

	$query->order($db->escape($orderCol . ' ' . $orderDirn));
	

	return $query;
  }

}