<?php

	/********************************************************************
	Cartproductfeed Extract Controller
		Feed Product info to RapidCart
		Copyright 2015 Purple Turtle Productions. All rights reserved.
		license	GNU General Public License version 3 or later; see GPLv3.txt
	By: Keneto
	2015-01
	********************************************************************/

defined('_JEXEC') or die;

//The extract AJAX should never need to display the define, so define it as blank
define('FEED_PLUGIN_VERSION', '' );

require_once dirname(__FILE__) . '/../core/data/feedcore.php';

class CartproductfeedControllerExtract extends JControllerAdmin {

	function checkInput() {

		global $pfcore;

		$input = JFactory::getApplication()->input;

		$this->token = $input->getString('token', '');
		$this->request = $input->getString('request', '');
		$this->product_limit_low = $input->getInt('product_limit_low', 0);
		$this->product_limit_high = $input->getInt('product_limit_high', 0);
		$this->specifier = $input->getString('specifier', '');

		$error = 0;
		if (strlen($this->token) == 0)
			$error = 1;
		if (strlen($this->request) == 0)
			$error = 2;

		$saved_token = $pfcore->settingGet('cp_rapidcarttoken');
		if (strlen($saved_token) == 0)
			$saved_token = $pfcore->settingGet('cp_licensekey');
		if (strlen($saved_token) == 0)
			$error = 5;
		else if ($this->token != $saved_token)
			$error = 4;

		//Make Specifier safe from SQL injection
		if (strlen($this->specifier) > 0) {
			$specifierArray = explode(',', $this->specifier);
			foreach($specifierArray as &$item)
				$item = (int) $item;
			$this->specifier = implode(',', $this->specifier);
		}

		//Set limit
		$this->limit = '';
		if ($this->product_limit_low > 0 || $this->product_limit_high > 0)
			$this->limit = ' LIMIT ' . $this->product_limit_low . ', ' . $this->product_limit_high;

		//Result
		$this->result = new stdClass();
		$this->result->version = 2;
		$this->result->error = $error;
		$this->result->endOfResults = false;
		$this->result->results = array();

		if ($this->result->error > 0) {
		//	echo json_encode($this->result);
		//	JFactory::getApplication()->close();
		}

	}

	function hcategories() {

		$this->checkInput();

		if ($this->result->error > 0) {
			echo json_encode($this->result);
			JFactory::getApplication()->close();
			return;
		}

		$query = '
			SELECT category_id as id, category_name as title, category_parent_id as parent_category, category_type
			FROM #__hikashop_category category' . 
			$this->limit;

		$this->resolveQuery($query);

		echo json_encode($this->result);
		JFactory::getApplication()->close();

	}

	function hlinks() {

		$this->checkInput();

		if ($this->result->error > 0) {
			echo json_encode($this->result);
			JFactory::getApplication()->close();
			return;
		}

		$query = '
			SELECT *
			FROM #__hikashop_product_category
			' . $this->limit;

		$this->resolveQuery($query);

		echo json_encode($this->result);
		JFactory::getApplication()->close();

	}

	function hproducts() {

		$db = JFactory::getDBO();

		$this->checkInput();

		if ($this->result->error > 0) {
			echo json_encode($this->result);
			JFactory::getApplication()->close();
			return;
		}

		$query = '
			SELECT product.*, price.price_value, currency.currency_symbol
			FROM #__hikashop_product product
			LEFT JOIN #__hikashop_price as price on (price.price_product_id = product.product_id)
			LEFT JOIN #__hikashop_currency currency on (currency.currency_id = price.price_currency_id)
			' . $this->limit;

		$this->resolveQuery($query);

		//Load extra data
		if(@include_once(rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_hikashop'.DS.'helpers'.DS.'helper.php')){ 

			foreach($this->result->results as $result) {

				//Attributes
				$attributes = array();
				$productClass = hikashop_get('class.product');
				$product = $productClass->get($result->product_id);

				$attributes['stock_quantity'] = $product->product_quantity;
				$attributes['handle'] = $product->product_code;
				$attributes['product_parent_id'] = $product->product_parent_id;
				$attributes['sale_price_dates_from'] = $product->product_sale_start;
				$attributes['sale_price_dates_to'] = $product->product_sale_end;
				$attributes['link'] = JURI::base() . 'index.php?option=com_hikashop&ctrl=product&task=show&cid=' . $result->product_id;

				$db->setQuery('SELECT file_path FROM #__hikashop_file WHERE file_ref_id=' . $result->product_id . ' ORDER BY file_ordering');
				$image = $db->loadResult();
				if (isset($image))
					$image = JURI::base() . 'images/com_hikashop/upload/' . $image;
				else
					$image = '';
				$attributes['feature_imgurl'] = $image;

				$result->attributes = $attributes;

				//Prices
				$db->setQuery('
					SELECT price.price_value, currency.currency_symbol
					FROM #__hikashop_price as price
					LEFT JOIN #__hikashop_currency currency on (currency.currency_id = price.price_currency_id)
					WHERE price.price_product_id = ' . $result->product_id);
				$result->prices = $db->loadObjectList();

			}

		}

		echo json_encode($this->result);
		JFactory::getApplication()->close();

	}

	function init() {

		$this->checkInput();

		if ($this->result->error > 0) {
			echo json_encode($this->result);
			JFactory::getApplication()->close();
			return;
		}

		echo json_encode($this->result);
		JFactory::getApplication()->close();

	}

	function resolveQuery($query) {

		$db = JFactory::getDBO();

		$db->setQuery($query);

		if ($this->product_limit_low + $this->product_limit_high > 0)
			$query .=  ' LIMIT ' . (int) $this->product_limit_low . ', ' . (int) $this->product_limit_high;

		$this->result->results = $db->loadObjectList();
		$this->result->resultCount = count($this->result->results);

		return $this->result;

	}


}